#include "mythcdrom.h"
#include <errno.h>
#include <fcntl.h>
#include "mythcontext.h"
#include "mythmediamonitor-darwin.h"

#define ASSUME_WANT_AUDIO 1

bool MythCDROMDarwin::openDevice()
{
    // Sanity check
    if (isDeviceOpen())
        return true;
 
    m_DeviceHandle = open(QString("/dev/r").append(m_DevicePath), O_RDONLY | O_NONBLOCK);
    
    return isDeviceOpen();
}

MediaError MythCDROMDarwin::eject(bool open_close)
{
    if (open_close)
    {
        VERBOSE(VB_IMPORTANT, "MythCDROMDarwin::eject - not yet implemented");
        return MEDIAERR_UNSUPPORTED;
    }
    else
        return MEDIAERR_UNSUPPORTED;
}

bool MythCDROMDarwin::mediaChanged()
{  
    // Not implemented
    return false;
}

bool MythCDROMDarwin::checkOK()
{
    // Not implemented
    return true;
}

// Helper function, perform a sanity check on the device
MediaError MythCDROMDarwin::testMedia()
{
    // cout << "MythCDROMDarwin::testMedia - ";
    bool OpenedHere = false;
    if (!isDeviceOpen()) 
    {
        // cout << "Device is not open - ";
        if (!openDevice()) 
        {
            // cout << "failed to open device - ";
            if (errno == EBUSY)
            {
                // cout << "errno == EBUSY" << endl;
                return isMounted(true) ? MEDIAERR_OK : MEDIAERR_FAILED;
            } 
            else 
            { 
                return MEDIAERR_FAILED; 
            }
        }
        // cout << "Opened it - ";
        OpenedHere = true;
    }

    // Be nice and close the device if we opened it, otherwise it might be locked when the user doesn't want it to be.
    if (OpenedHere)
        closeDevice();

    return MEDIAERR_OK;
}

MediaStatus MythCDROMDarwin::checkMedia()
{
    if (m_Status == MEDIASTAT_UNKNOWN)
    {
        // Set our media type
        m_MediaType = MediaTypeForBSDName(m_DevicePath);
        // Update the DVD device path setting (used by the DVD Player app) - only update for the session, 
        // on Mac OS X device paths are very changeable.
        if (m_MediaType == MEDIATYPE_DVD)
        {
            gContext->OverrideSettingForSession("DVDDeviceLocation", QString("/dev/r").append(m_DevicePath));
        }
        // Pretend we're not mounted to setStatus emits the signal
        m_Status = MEDIASTAT_NOTMOUNTED;
        return setStatus(MEDIASTAT_USEABLE, false);
    }
    else
        return m_Status;
}

MediaError MythCDROMDarwin::lock() 
{
    MediaError ret = MythMediaDevice::lock();
//    if (ret == MEDIAERR_OK)
//        ioctl(m_DeviceHandle, CDIOCPREVENT);

    return ret;
}

MediaError MythCDROMDarwin::unlock() 
{
    if (isDeviceOpen() || openDevice()) 
    { 
        //cout <<  "Unlocking CDROM door" << endl;
//        ioctl(m_DeviceHandle, CDIOCALLOW);
    }
    else
    {
        VERBOSE(VB_GENERAL, "Failed to open device, CDROM try will remain "
                            "locked.");
    }

    return MythMediaDevice::unlock();
}
